/*
 * Copyright (C) 2003 Antonio Fasolato <Antonio.Fasolato@poste.it>
 */

#include "potracegui.h"
#include "errordialog.h"
#include "configuration.h"

#include <qlabel.h>
#include <qfiledialog.h>
#include <qprocess.h>
#include <qstringlist.h>
#include <qmessagebox.h>

#include <kmainwindow.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kpushbutton.h>
#include <kdialogbase.h>

#include <iostream>
using namespace std;

PotraceGui::PotraceGui()
    : PotraceGuiBase(), filename(), outputPreview(), options()
{
}

PotraceGui::~PotraceGui()
{
 	if(outputPreview.isRunning())
 		outputPreview.tryTerminate();
}

void PotraceGui::selectImage(){
	KFileDialog *d=new KFileDialog(QString::null,"*.pbm *.pgm *.ppm *.bmp|Bitmap images",this,"Select an image",true);

	d->exec();
	if(d->result()==QDialog::Accepted) {
		filename=d->selectedFile();
		QPixmap *pix=new QPixmap(filename);
		preview->setPixmap(*pix);
		options.clear();
		setOptionsBTN->setEnabled(true);
		traceBTN->setEnabled(true);
		if(outputPreview.isRunning())
			outputPreview.tryTerminate();
		allDone->setText("New image selected");
	}
}

void PotraceGui::setOptions(){
	OptionDialog *opt=new OptionDialog(options);

	opt->exec();
	allDone->setText("Options changed: press trace!");
}

void PotraceGui::trace(){
	//controllare pagesize manuale
	if(options["outputFileName"]=="") {
		ErrorDialog err("No otput file","You haven't specified an output file.\nPlease check the options page.");
		err.exec();
	}
	else {
		QStringList commandLine=createCommandLine();

		QProcess *p=new QProcess(this);
		p->setArguments(commandLine);

		if(!(p->start())) {
			ErrorDialog err("Error!","The process couldn't be started\nThe process executed was:\n"+commandLine.join(" "));
			err.exec();
			return;
		}

		if(p->exitStatus()!=0) {
			QString tmp("Error during the process:");
			tmp+="\nStandard output:\n";
			QString t=p->readLineStdout();
			while(t!=QString::null) {
				tmp+= t+"\n";
				t=p->readLineStdout();
			}
			tmp+="\nStandard error:\n";
			t=p->readLineStderr();
			while(t!=QString::null) {
				tmp+=t+"\n";
				t=p->readLineStderr();
			}

			QStringList l=p->arguments();
			tmp+="\nExecuted:\n"+l.join(" ");

			ErrorDialog err("Errore!",tmp);
			err.exec();

			delete p;
			return;
		}

		if(Conf().preview==true) {
			if(outputPreview.isRunning())
				outputPreview.tryTerminate();

			QStringList arg=QStringList::split(" ",Conf().viewer);
			arg+=options["outputFileName"];
			outputPreview.setArguments(arg);
			outputPreview.start();
		}
		allDone->setText("Image traced!");

		delete p;
	}
}

QStringList PotraceGui::createCommandLine(){
	QStringList toReturn=QStringList::split(" ",Conf().potrace);
	toReturn+=filename;

	//input
	if(options["blackLevel"]!="0.5")   //only if it is not the default value
		toReturn+="-k"+options["blackLevel"];
	if(options["invertInput"]=="y")
		toReturn+="-i";

	//output
	toReturn+="-o";
	toReturn+=options["outputFileName"];
	if(options["outputFormat"]=="eps") {                   //eps format
		toReturn+="-e";
		toReturn+="-P"+options["pageSize"];
		toReturn+="-C"+options["foregroundBtn"];
		if(options["opaque"]=="y") {
			toReturn+="--opaque";
			//toReturn+="--fillcolor"+options["backgroundBtn"];
		}
		if(options["optimizedNumericalCode"]=="n")
			toReturn+="-q";
		if(options["compressionLevel"]=="1")
			toReturn+="-c";
		if(options["compressionLevel"]=="2")
			toReturn+="-2";
		if(options["compressionLevel"]=="3")
			toReturn+="-3";
	}
	else if(options["outputFormat"]=="postscript") {      //ps format
		toReturn+="-p";
		toReturn+="-P"+options["pageSize"];
		toReturn+="-C"+options["foregroundBtn"];
		if(options["opaque"]=="y") {
			toReturn+="--opaque";
			//toReturn+="--fillcolor"+options["backgroundBtn"];
		}
		if(options["optimizedNumericalCode"]=="n")
			toReturn+="-q";
		if(options["compressionLevel"]=="1")
			toReturn+="-c";
		if(options["compressionLevel"]=="2")
			toReturn+="-2";
		if(options["compressionLevel"]=="3")
			toReturn+="-3";
	}
	else if(options["outputFormat"]=="SVG") {            //SVG format
		toReturn+="-s";
		toReturn+="-C"+options["foregroundBtn"];
		if(options["opaque"]=="y") {
			toReturn+="--opaque";
			//toReturn+="--fillcolor"+options["backgroundBtn"];
		}
		if(options["SVGGroup"]=="y")
			toReturn+="--group";
	}
	else if(options["outputFormat"]=="Gimp path") {            //Gimp format
		toReturn+="-bgimppath";
	}
	else if(options["outputFormat"]=="Xfig") {            //Xfig format
		toReturn+="-bxfig";
		toReturn+="-C"+options["foregroundBtn"];
		if(options["opaque"]=="y") {
			toReturn+="--opaque";
			//toReturn+="--fillcolor"+options["backgroundBtn"];
		}
	}
	else {                                                               //pgm format
		toReturn+="-g";
		toReturn+="-G"+options["antialiasGamma"];
		if(options["opaque"]=="y") {
			toReturn+="--opaque";
			//toReturn+="--fillcolor"+options["backgroundBtn"];
		}
	}

	//Algorithm
	toReturn+="-z"+options["policy"];
	toReturn+="-t"+options["despeckle"];
	toReturn+="-a"+options["cornerThreshold"];
	toReturn+="-O"+options["optimizationTolerance"];
	toReturn+="-u"+options["outputQuantization"];
	if(options["curveOptimization"]=="n")
		toReturn+="-n";

	//Transformation
	if(options["width"]!="0")
		toReturn+="-W"+options["width"];
	if(options["height"]!="0")
		toReturn+="-H"+options["height"];
	if(options["stretch"]!="1")
		toReturn+="-S"+options["stretch"];
	if(options["rotation"]!="0")
		toReturn+="-A"+options["rotation"];

	//Resolution
	if(options["resolution"]!="0" && options["resolution"]!="0x0")
		toReturn+="-r"+options["resolution"];

	if(options["margins"]!="0" && options["margins"]!="0x0x0x0"){
		if(options["margins"].find('x')==-1)
			toReturn+="-M"+options["margins"];
		else {
			QString tmp=options["margins"];
			toReturn+="-L"+tmp.section('x',0,0);
			toReturn+="-R"+tmp.section('x',1,1);
			toReturn+="-T"+tmp.section('x',2,2);
			toReturn+="-B"+tmp.section('x',3,3);
		}
	}

	return toReturn;
}

void PotraceGui::settings(){
	Settings *s=new Settings();
	int i=s->exec();
	if(i==QDialog::Accepted && outputPreview.isRunning())
		outputPreview.tryTerminate();
}

#include "potracegui.moc"
